<?php

/**
 * ShopConstruct Admin
 *
 * @class       Shop_CT_Admin
 * @category    Admin
 * @package     ShopConstruct/Admin
 * @version     2.3
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Shop_CT_Admin class.
 */
class Shop_CT_Admin
{

    /**
     * Current page of admin panel
     */
    public $current_page = "";

    /**
     * Admin menus instance
     * @var Shop_CT_Admin_Menus
     */
    public $menus = null;

    /**
     * Assets for shop_ct pages
     * @var Shop_CT_Admin_Assets
     */
    public $assets = null;

    /**
     * Constructor.
     */
    public function __construct()
    {
        add_action('init', array($this, 'includes'));
    }

    /**
     * Include any classes we need within admin.
     */
    public function includes()
    {

        include_once('list-table/class-shop-ct-list-table.php');
        include_once('list-table/class-shop-ct-list-table-attribute-taxonomies.php');
        include_once('list-table/class-shop-ct-list-table-order.php');
        include_once('list-table/class-shop-ct-list-table-reviews.php');
        include_once('list-table/class-shop-ct-list-table-products.php');

        $this->assets = apply_filters('shop_ct_admin_assets', $this->assets);
        $this->assets = apply_filters('shop_ct_admin_scripts', $this->assets);
        $this->assets = apply_filters('shop_ct_admin_styles', $this->assets);

        $this->init();


    }

    public function init()
    {
        $this->assets = new Shop_CT_Admin_Assets();

        $this->menus = new Shop_CT_Admin_Menus();
    }


    public static function gutenberg_block()
    {
        if (!function_exists('register_block_type')) {
            return;
        }

        wp_register_script(
            'shop_ct_gutenberg_block',
            SHOP_CT()->plugin_url() . '/assets/js/admin/admin.blocks.js',
            array('wp-blocks', 'wp-element', 'wp-components')
        );
        wp_register_style(
            'shop_ct_gutenberg_block',
            SHOP_CT()->plugin_url() . '/assets/css/admin.blocks.css',
            array('wp-edit-blocks')
        );

        $products = Shop_CT_Product::all();
        $categories = Shop_CT_Product_Category::all();

        $productOptions = array();
        $productMetas = array();
        $categoryOptions = array();
        $categoryMetas = array();

        if (!empty($products)) {
            foreach ($products as $product) {
                $productOptions[] = [
                    'value' => $product->get_id(),
                    'label' => $product->get_title(),
                ];
                $productMetas[$product->get_id()] = array(
                    'title' => $product->get_title(),
                );
            }
        }

        if (!empty($categories)) {
            foreach ($categories as $category) {
                $categoryOptions[] = [
                    'value' => $category->get_id(),
                    'label' => $category->get_name(),
                ];
                $categoryMetas[$category->get_id()] = array(
                    'title' => $category->get_name(),
                );
            }
        }


        wp_localize_script('shop_ct_gutenberg_block', 'shopCTBlockI10n', array(
            'products' => $productOptions,
            'productMetas' => $productMetas,
            'categories' => $categoryOptions,
            'categoryMetas' => $categoryMetas
        ));

        register_block_type('shop-ct/product', array(
            'editor_script' => 'shop_ct_gutenberg_block',
            'editor_style' => 'shop_ct_gutenberg_block',
        ));
    }

    public static function gutenberg_block_categories($categories, $post)
    {
        if ($post->post_type !== 'post') {
            return $categories;
        }
        return array_merge(
            $categories,
            array(
                array(
                    'slug' => 'shop-ct',
                    'title' => __('ShopConstruct', 'photo-portfolio'),
                ),
            )
        );
    }

}