<?php

/**
 * Class Shop_CT_Display_Settings
 */
class Shop_CT_Display_Settings extends Shop_CT_Settings
{

    public $id = 'display';

    /** @var array Pages Where Shop items are displayed */
    public $shop_pages = array();

    /** @var  array */
    private $pages;

    /** @var  array */
    private $posts;

    /** @var  array */
    private $shop_ids;

    /** @var  array */
    private $page_types;

    /**
     * Shop_CT_Display_Settings constructor.
     */
    public function __construct()
    {
        $this->init();
    }

    public function init()
    {
        $this->shop_pages = $this->get_option('shop_pages', array());
        $this->pages = get_pages();
        $this->posts = get_posts();
        $this->shop_ids = array();


        if (is_array($this->shop_pages) && !empty($this->shop_pages)) {
            foreach ($this->shop_pages as $shop_page) {
                array_push($this->shop_ids, $shop_page['post_id']);
            }
        }
        $this->page_types = apply_filters('shop_ct_shop_page_types', array(
            'single_product' => __('Single Product', 'shop_ct'),
            'single_category' => __('Single Category', 'shop_ct'),
            'all_products' => __('All Products', 'shop_ct'),
            'all_categories' => __('All Categories', 'shop_ct'),
        ));
        $this->display();
    }

    /**
     * Display the page
     * @return string
     */
    public function display()
    {
        return Shop_CT_Template_Loader::get_template_buffer('admin/settings/display/show.php', array(
            'pages' => $this->pages,
            'posts' => $this->posts,
            'shop_ids' => $this->shop_ids,
            'shop_pages' => $this->shop_pages,
            'page_types' => $this->page_types
        ));
    }

}